# -*- coding: utf-8 -*-

from era.noyau.dpatterns import Observer, Observable, FluxListener, ZoneListener
from era.noyau.erreurs import EmptyNameError, ExistingZoneError, TrigrammeError
from era import version

class LibraryStore(Observable):

    def __init__(self, service_path=None, extremite_path=None):
        Observable.__init__(self)
        # charger à partir de service_path, extremite_path
        self.version = version
        self.services = []
        self.groups = []
        self.extremites = {}
        self.tags = {}
        self.ranges = {}
        self.rules = None
        self.user_groups = {}
        self.options = {'netbios':1}

    def set_version(self, cur_version):
        """la version des modeles xml"""
        self.version = float(cur_version)

    def get_version(self):
        """la version des modeles xml"""
        return self.version
    
    def set_options(self, options):
        """@param options: dictionnaire des options"""
        self.options.update(options)
        
    def  add_range(self, range):
        self.ranges.append(range)
        self.notify_observers()
        
    def add_service(self, service):
        # Vérifier que ce service est valide ..., sinon exception
        self.services.append(service)
        self.notify_observers()

    def add_group(self, group):
        # Vérifier que ce groupe est valide ..., sinon exception
        self.groups.append(group)
        self.notify_observers()

    def add_extremite(self, extr):
        # Vérifier que cette extremite est valide ..., sinon exception
        self.extremites[extr.name] = extr
        # self.extremites.append(extr)
        self.notify_observers()

    def clear(self):
        self.services = []
        self.groups = []
        self.rules = None
        self.tags = {}
        self.ranges = {}
        self.extremites = {}
        self.user_groups = {}
        self.options['netbios'] = 1
        self.notify_observers()

    def clear_tags(self):
        self.tags={}

    def remove_extremites(self, zone):
        """Retire l'ensemble des extrémités liées à une zone.
        """
        for e in self.get_zone_extremites(zone):
            del self.extremites[e.name]
        

    def get_zone_extremites(self, zone):
        """Renvoie la liste des extremites d'une zone
        """
        return [ext for ext in library_store.extremites.values() if ext.zone == zone]

    def save(self, stream, indent = 0):
        """Méthode qui sauvegarde, au format interne, les données
        du library_store
        """
        if self.rules:
            stream.write('%s<include>%s</include>\n'% (" "*indent, self.rules) )
        
        stream.write('%s<services>\n'% (" "*indent))
        for service in self.services:
            # XXX : Trick pour pas sauver en double lors d'héritage
            if not service.is_inherited():
                service.save(stream, indent+4)
        for group in self.groups:
            if not group.is_inherited():
                group.save(stream, indent+4)
        stream.write('%s</services>\n'% (" "*indent))
        stream.write('%s<extremites>\n'% (" "*indent))
        for extr in self.extremites.values():
            if not extr.is_inherited():
                extr.save(stream, indent+4)
        stream.write('%s</extremites>\n'% (" "*indent))
        stream.write('%s<ranges>\n'% (" "*indent))
        for timerange in self.ranges.values():
            if not timerange.is_inherited():
                timerange.save(stream, indent+4)
        stream.write('%s</ranges>\n'% (" "*indent))
        stream.write('%s<user_groups>\n'% (" "*indent))
        for user_group in self.user_groups.values():
            if not user_group.is_inherited():
                user_group.save(stream, indent+4)
        stream.write('%s</user_groups>\n'% (" "*indent))
        

# print "Creating library store ..."
library_store = LibraryStore()
# print "ID = %d"%id(library_store)

